using System;
using System.Collections;
using System.Data;
using System.IO;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using LoMaN.IO;


namespace CPFLite
{
	public class ReaderThread
	{
		#region Members

		// Main thread sets this event to stop worker thread:
		ManualResetEvent m_EventStop;

		// Worker thread sets this event when it is stopped:
		ManualResetEvent m_EventStopped;

		// Reference to main form used to make syncronous user interface calls:
		CPFLite m_form;

		#endregion

		#region Default variables

		//These hold the resistor values for current calculation.
		public static double iA1 = 0;
		public static double iA2 = 0;
		public static double iA3 = 0;
		public static double iA4 = 0;

		//These determine whether the current calc checkboxes are checked.
		public static bool bA1 = false;
		public static bool bA2 = false;
		public static bool bA3 = false;
		public static bool bA4 = false;

		//This stores the Vref value with a default of 5V.
		public static double dVref = 5;

		//This holds the name of the serial port to be used.
		public static string sComPort = "COM1";

		//This holds the baud-rate. Default is 9600..
		//This was added to try to compensate for uncalibrated pic's.
		public static uint iBaudRate = 9600;

		//This determines whether the thread has been sent a stop request.
		public static bool bStopThread = false;

		#endregion

		#region Instantiators

		//This creates the SerialStream device from SerialStream.cs
		public static SerialStream ss = new SerialStream();

		//This sets up the ASCII encoding formatter needed to get the
		//text out of the serial port buffer.
		ASCIIEncoding enc = new ASCIIEncoding();

		#endregion

		#region Port Maintenance

		//This sets the parameters for the serial port and opens
		//the port for use. The "Sleep" delay is used to ensure that
		//the PIC is reset between start/stop cycles.
		private void OpenPort()
		{
			Thread.Sleep(100);
			ss.Open(sComPort);
			ss.SetPortSettings(iBaudRate,SerialStream.FlowControl.Hardware, SerialStream.Parity.None, 8, SerialStream.StopBits.One);
			ss.SetTimeouts(50,1,50,1,50);

		}


		//This closes the serial port and purges the buffer if needed.
		private void ClosePort()
		{
			ss.Flush();
			ss.Close();
		}

		#endregion

		public ReaderThread(ManualResetEvent eventStop, ManualResetEvent eventStopped, CPFLite form)
		{
			bStopThread = true;
			//ClosePort();
			m_EventStop = eventStop;
			m_EventStopped = eventStopped;
			m_form = form;
		}


		// Function runs in worker thread and emulates long process.
		public void Run()
		{
			OpenPort();
			bStopThread = false;

			if (ss.Dsr == true)
			{
				ReadConvert();
			}

			//Close the port when finished.
			ClosePort();

			// Make synchronous call to main form
			// to inform it that thread finished
			m_form.Invoke(m_form.m_DelegateThreadFinished, null);
		}

		private void ReadConvert()
		{
			while(bStopThread == false)
			{
				//This sets up the string array that is returned.
				string[] sVals = new string[8];

				//Create a byte array to store the incoming line from the serial buffer.
				byte[] bContents = new byte[32];

				//Read from the buffer until it contains something.
				//Store the resulting 32 characters of data into the byte array.
				while(bContents[31] != 10)
				{
					ss.Read(bContents,0,32);
				}

				//Store the timestamp (or the "-"'s if in fast mode) as the first string
				//in the returned array.
				sVals[0] = enc.GetString(bContents,0,10);

				//Here we use the ASCII encoder to convert the byte array into a readable ASCII string.
				//Next, convert that to a "double" value.
				double dV1 = Convert.ToDouble(enc.GetString(bContents,11,4));
				double dV2 = Convert.ToDouble(enc.GetString(bContents,16,4));
				double dV3 = Convert.ToDouble(enc.GetString(bContents,21,4));
				double dV4 = Convert.ToDouble(enc.GetString(bContents,26,4));

				//Then, multiply by Vref, then divide by the ADC-Max value of 4095 to get the true voltage.
				dV1 = dV1 * dVref / 4095;
				dV2 = dV2 * dVref / 4095;
				dV3 = dV3 * dVref / 4095;
				dV4 = dV4 * dVref / 4095;

				//Here we check if the current resistor check-boxes are checked.
				//If so, divide V by R to get A (Ohm's law).
				if (bA1 == true) {dV1 = dV1 / iA1;}
				if (bA2 == true) {dV2 = dV2 / iA2;}
				if (bA3 == true) {dV3 = dV3 / iA3;}
				if (bA4 == true) {dV4 = dV4 / iA4;}

				//Put the voltages (rounded to 3 decimal places) into the string array.
				sVals[1] = Math.Round(dV1,3).ToString();
				sVals[2] = Math.Round(dV2,3).ToString();
				sVals[3] = Math.Round(dV3,3).ToString();
				sVals[4] = Math.Round(dV4,3).ToString();

				//Calculate power and efficiency
				double dP1 = (dV1 * dV2);
				double dP2 = (dV3 * dV4);
				double dEff = (dP2/dP1)*100;

				//Put the power and efficiency into the string array
				sVals[5] = Math.Round(dP1,3).ToString();
				sVals[6] = Math.Round(dP2,3).ToString();
				sVals[7] = Math.Round(dEff,2).ToString();

				// Make synchronous call to main form.
				// MainForm.AddString function runs in main thread.
				// (To make asynchronous call use BeginInvoke)
				m_form.Invoke(m_form.m_DelegateAddString, new Object[] {sVals});

				// check if thread is cancelled
				if ( m_EventStop.WaitOne(0, true) )
				{
					ClosePort();
					// inform main thread that this thread stopped
					m_EventStopped.Set();

					return;
				}
			}
		}

	}
}
